@@ -1257,7 +1257,7 @@ class Cloud(object):
                 time.sleep(3)
 
                 mopts_ = salt.config.DEFAULT_MINION_OPTS
-                conf_path = '/'.join(__opts__['conf_file'].split('/')[:-1])
+                conf_path = '/'.join(self.opts['conf_file'].split('/')[:-1])
                 mopts_.update(
                     salt.config.minion_config(
                         os.path.join(conf_path,
