@@ -561,7 +561,7 @@ def create_disk_from_distro(vm_, linode_id, swap_size=None):
     r'''
     Creates the disk for the Linode from the distribution.
 
-    vm_
+    vm\_
         The VM profile to create the disk for.
 
     linode_id
@@ -588,12 +588,10 @@ def create_disk_from_distro(vm_, linode_id, swap_size=None):
             'The Linode driver requires a password.'
         )
 
-    distribution_id = get_distribution_id(vm_)
-
     kwargs.update({'LinodeID': linode_id,
-                   'DistributionID': distribution_id,
+                   'DistributionID': get_distribution_id(vm_),
                    'Label': vm_['name'],
-                   'Size': get_disk_size(vm_, swap_size)})
+                   'Size': get_disk_size(vm_, swap_size, linode_id)})
 
     result = _query('linode', 'disk.createfromdistribution', args=kwargs)
 
@@ -604,7 +602,7 @@ def create_swap_disk(vm_, linode_id, swap_size=None):
     r'''
     Creates the disk for the specified Linode.
 
-    vm_
+    vm\_
         The VM profile to create the swap disk for.
 
     linode_id
@@ -633,7 +631,7 @@ def create_private_ip(vm_, linode_id):
     r'''
     Creates a private IP for the specified Linode.
 
-    vm_
+    vm\_
         The VM profile to create the swap disk for.
 
     linode_id
@@ -746,15 +744,14 @@ def get_datacenter_id(location):
     return avail_locations()[location]['DATACENTERID']
 
 
-def get_disk_size(vm_, swap):
+def get_disk_size(vm_, swap, linode_id):
     r'''
     Returns the size of of the root disk in MB.
 
-    vm_
+    vm\_
         The VM to get the disk size for.
     '''
-    vm_size = get_vm_size(vm_)
-    disk_size = vm_size
+    disk_size = get_linode(kwargs={'linode_id': linode_id})['TOTALHD']
     return config.get_cloud_config_value(
         'disk_size', vm_, __opts__, default=disk_size - swap
     )
@@ -764,7 +761,7 @@ def get_distribution_id(vm_):
     r'''
     Returns the distribution ID for a VM
 
-    vm_
+    vm\_
         The VM to get the distribution ID for
     '''
     distributions = _query('avail', 'distributions')['DATA']
@@ -893,7 +890,7 @@ def get_password(vm_):
     r'''
     Return the password to use for a VM.
 
-    vm_
+    vm\_
         The configuration to obtain the password from.
     '''
     return config.get_cloud_config_value(
@@ -949,7 +946,7 @@ def get_pub_key(vm_):
     r'''
     Return the SSH pubkey.
 
-    vm_
+    vm\_
         The configuration to obtain the public key from.
     '''
     return config.get_cloud_config_value(
@@ -961,11 +958,11 @@ def get_swap_size(vm_):
     r'''
     Returns the amoutn of swap space to be used in MB.
 
-    vm_
+    vm\_
         The VM profile to obtain the swap size from.
     '''
     return config.get_cloud_config_value(
-        'wap', vm_, __opts__, default=128
+        'swap', vm_, __opts__, default=128
     )
 
 
@@ -973,7 +970,7 @@ def get_vm_size(vm_):
     r'''
     Returns the VM's size.
 
-    vm_
+    vm\_
         The VM to get the size for.
     '''
     vm_size = config.get_cloud_config_value('size', vm_, __opts__)
