@@ -448,7 +448,7 @@ def latest_version(*names, **kwargs):
     return ret
 
 # available_version is being deprecated
-available_version = latest_version
+available_version = salt.utils.alias_function(latest_version, 'available_version')
 
 
 def upgrade_available(name):
@@ -755,10 +755,11 @@ def check_db(*names, **kwargs):
         __context__['pkg._avail'] = avail
 
     ret = {}
-    repoquery_cmd = repoquery_base + ' {0}'.format(" ".join(names))
-    provides = sorted(
-        set(x.name for x in _repoquery_pkginfo(repoquery_cmd))
-    )
+    if names:
+        repoquery_cmd = repoquery_base + ' {0}'.format(" ".join(names))
+        provides = sorted(
+            set(x.name for x in _repoquery_pkginfo(repoquery_cmd))
+        )
     for name in names:
         ret.setdefault(name, {})['found'] = name in avail
         if not ret[name]['found']:
@@ -1744,7 +1745,7 @@ def group_install(name,
 
     return install(pkgs=pkgs, **kwargs)
 
-groupinstall = group_install
+groupinstall = salt.utils.alias_function(group_install, 'groupinstall')
 
 
 def list_repos(basedir=None):
@@ -1914,6 +1915,14 @@ def mod_repo(repo, basedir=None, **kwargs):
             del repo_opts[key]
             todelete.append(key)
 
+    # convert disabled=True to enabled=0 from pkgrepo state
+    if 'disabled' in repo_opts:
+        kw_disabled = repo_opts['disabled']
+        if kw_disabled is True or str(kw_disabled).lower() == 'true':
+            repo_opts['enabled'] = 0
+        del repo_opts['disabled']
+        todelete.append('disabled')
+
     # Add baseurl or mirrorlist to the 'todelete' list if the other was
     # specified in the repo_opts
     if 'mirrorlist' in repo_opts:
